from functools import lru_cache
from typing import List

from aws_cdk.aws_lambda import Code, LayerVersion, Runtime
from aws_cdk.core import Stack, AssetHashType, BundlingOptions, BundlingDockerImage


class Layer(LayerVersion):
    def __init__(self, scope: Stack, name: str):
        self.__scope = scope

        super().__init__(
            scope=scope,
            id=name,
            code=Code.from_asset(
                self.get_source_path(),
                asset_hash_type=AssetHashType.BUNDLE,
                bundling=BundlingOptions(
                    image=BundlingDockerImage.from_registry('python:3.9'),
                    command=[
                        'bash', '-c',
                        (
                            'pip install -r requirements.txt -t /asset-output/python && '
                            'find /asset-output -type f -name "*.py[co]" -delete && '
                            'find /asset-output -type d -name "__pycache__" -delete'
                        )
                    ]
                )
            ),
            compatible_runtimes=self.runtimes(),
            layer_version_name=name,
        )

    @lru_cache
    def get_source_path(self) -> str:
        """
        Returns path to layer source.

        :return: Path to layer source.
        """
        from .source import root
        return root

    def runtimes(self) -> List[Runtime]:
        """
        Available runtimes for lambda functions.

        :return: List of available runtimes.
        """
        return [
            Runtime.PYTHON_3_6,
            Runtime.PYTHON_3_7,
            Runtime.PYTHON_3_8
        ]



import os
import sys

from aws_cdk.core import App

"""
Import main stack.
"""

ROOT_PATH = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
sys.path.append(ROOT_PATH)
from b_elasticsearch_layer_test.testing_infrastructure import TestingInfrastructure

"""
Create CDK app.
"""

app = App()
TestingInfrastructure(app)
app.synth()



from aws_cdk.aws_lambda import Function, Code, Runtime
from aws_cdk.core import Stack

from b_elasticsearch_layer.layer import Layer as ElasticsearchLayer


class TestingInfrastructure(Stack):
    def __init__(self, scope: Stack):
        super().__init__(
            scope=scope,
            id=f'TestingStack',
            stack_name=f'TestingStack'
        )

        Function(
            scope=self,
            id='TestingFunction',
            code=Code.from_inline(
                'def handler(): return "Hello World!"'
            ),
            handler='index.handler',
            runtime=Runtime.PYTHON_3_6,
            layers=[ElasticsearchLayer(self, 'TestingElasticsearchLayer')]
        )



from setuptools import setup, find_packages

with open('README.md') as readme_file:
    README = readme_file.read()

with open('HISTORY.md') as history_file:
    HISTORY = history_file.read()

with open('VERSION') as file:
    VERSION = file.read()
    VERSION = ''.join(VERSION.split())

setup(
    name='b_elasticsearch_layer',
    version=VERSION,
    license='Apache License 2.0',
    packages=find_packages(exclude=[
        # Exclude virtual environment.
        'venv',
        # Exclude test source files.
        'b_elasticsearch_test'
    ]),
    description=(
        'AWS CDK based lambda layer including Elasticsearch client.'
    ),
    long_description=README + '\n\n' + HISTORY,
    long_description_content_type="text/markdown",
    include_package_data=True,
    install_requires=[
        "aws-cdk.core>=1.54.0,<2.0.0",
        "aws-cdk.aws_lambda>=1.54.0,<2.0.0",
    ],
    author='Laimonas Sutkus',
    author_email='laimonas.sutkus@biomapas.com',
    keywords='AWS CDK Lambda Layer Elasticsearch',
    url='https://github.com/biomapas/B.ElasticsearchLayer.git',
    classifiers=[
        'Programming Language :: Python :: 3',
        'Operating System :: OS Independent',
    ],
)
